@@ -82,14 +82,6 @@ public class JMXAccessorQueryTask extends JMXAccessorTask {
     // ------------------------------------------------------ protected Methods
 
 
-    /**
-     * Execute the specified command, based on the configured properties. The
-     * input stream will be closed upon completion of this task, whether it was
-     * executed successfully or not.
-     *
-     * @exception Exception
-     *                if an error occurs
-     */
     @Override
     public String jmxExecute(MBeanServerConnection jmxServerConnection)
         throws Exception {
@@ -105,10 +97,10 @@ public class JMXAccessorQueryTask extends JMXAccessorTask {
     /**
      * Call Mbean server for some mbeans with same domain, attributes.
      *  with <em>attributebinding=true</em> you can save all attributes from all found objects
-     * as your ant properties
-     * @param jmxServerConnection
-     * @param qry
-     * @return The query result
+     *
+     * @param jmxServerConnection Connection to the JMX server
+     * @param qry The query
+     * @return null (no error message to report other than exception)
      */
     protected String jmxQuery(MBeanServerConnection jmxServerConnection,
             String qry) {
@@ -143,57 +135,39 @@ public class JMXAccessorQueryTask extends JMXAccessorTask {
         return isError;
     }
 
+    protected void bindAttributes(MBeanServerConnection jmxServerConnection, String pname, ObjectName oname) {
+        try {
+            MBeanInfo minfo = jmxServerConnection.getMBeanInfo(oname);
+            MBeanAttributeInfo attrs[] = minfo.getAttributes();
+            Object value = null;
 
-    /**
-     * @param jmxServerConnection
-     * @param resultproperty
-     * @param pname
-     * @param oname
-     *
-     * @deprecated Unused. Will be removed in Tomcat 8.5.x
-     */
-    @Deprecated
-    protected void bindAttributes(MBeanServerConnection jmxServerConnection,
-            String resultproperty, String pname, ObjectName oname) {
-        bindAttributes(jmxServerConnection, pname, oname);
-    }
-
-    protected void bindAttributes(MBeanServerConnection jmxServerConnection,
-            String pname, ObjectName oname) {
-        if (jmxServerConnection != null && pname != null && oname != null ) {
-            try {
-                MBeanInfo minfo = jmxServerConnection.getMBeanInfo(oname);
-                MBeanAttributeInfo attrs[] = minfo.getAttributes();
-                Object value = null;
-
-                for (int i = 0; i < attrs.length; i++) {
-                    if (!attrs[i].isReadable())
-                        continue;
-                    String attName = attrs[i].getName();
-                    if (attName.indexOf('=') >= 0 || attName.indexOf(':') >= 0
-                            || attName.indexOf(' ') >= 0) {
-                        continue;
-                    }
-
-                    try {
-                        value = jmxServerConnection
-                                .getAttribute(oname, attName);
-                    } catch (Exception e) {
-                        if (isEcho())
-                            handleErrorOutput("Error getting attribute "
-                                    + oname + " " + pname + attName + " "
-                                    + e.toString());
-                        continue;
-                    }
-                    if (value == null)
-                        continue;
-                    if ("modelerType".equals(attName))
-                        continue;
-                    createProperty(pname + attName, value);
+            for (int i = 0; i < attrs.length; i++) {
+                if (!attrs[i].isReadable())
+                    continue;
+                String attName = attrs[i].getName();
+                if (attName.indexOf('=') >= 0 || attName.indexOf(':') >= 0
+                        || attName.indexOf(' ') >= 0) {
+                    continue;
                 }
-            } catch (Exception e) {
-                // Ignore
+
+                try {
+                    value = jmxServerConnection
+                            .getAttribute(oname, attName);
+                } catch (Exception e) {
+                    if (isEcho())
+                        handleErrorOutput("Error getting attribute "
+                                + oname + " " + pname + attName + " "
+                                + e.toString());
+                    continue;
+                }
+                if (value == null)
+                    continue;
+                if ("modelerType".equals(attName))
+                    continue;
+                createProperty(pname + attName, value);
             }
+        } catch (Exception e) {
+            // Ignore
         }
     }
 }
