@@ -18,11 +18,15 @@ package org.apache.catalina.connector;
 
 import java.io.IOException;
 import java.io.Reader;
+import java.nio.Buffer;
+import java.nio.ByteBuffer;
+import java.nio.CharBuffer;
+import java.nio.charset.Charset;
 import java.security.AccessController;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.ConcurrentMap;
 
 import javax.servlet.ReadListener;
 
@@ -32,7 +36,8 @@ import org.apache.coyote.ContainerThreadMarker;
 import org.apache.coyote.Request;
 import org.apache.tomcat.util.buf.B2CConverter;
 import org.apache.tomcat.util.buf.ByteChunk;
-import org.apache.tomcat.util.buf.CharChunk;
+import org.apache.tomcat.util.collections.SynchronizedStack;
+import org.apache.tomcat.util.net.ApplicationBufferHandler;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
@@ -44,22 +49,14 @@ import org.apache.tomcat.util.res.StringManager;
  * @author Remy Maucherat
  */
 public class InputBuffer extends Reader
-    implements ByteChunk.ByteInputChannel, CharChunk.CharInputChannel,
-               CharChunk.CharOutputChannel {
+    implements ByteChunk.ByteInputChannel, ApplicationBufferHandler {
 
     /**
      * The string manager for this package.
      */
-    protected static final StringManager sm =
-        StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(InputBuffer.class);
 
-
-    // -------------------------------------------------------------- Constants
-
-
-    public static final String DEFAULT_ENCODING =
-        org.apache.coyote.Constants.DEFAULT_CHARACTER_ENCODING;
-    public static final int DEFAULT_BUFFER_SIZE = 8*1024;
+    public static final int DEFAULT_BUFFER_SIZE = 8 * 1024;
 
     // The buffer can be used for byte[] and char[] reading
     // ( this is needed to support ServletInputStream and BufferedReader )
@@ -67,19 +64,24 @@ public class InputBuffer extends Reader
     public final int CHAR_STATE = 1;
     public final int BYTE_STATE = 2;
 
-    // ----------------------------------------------------- Instance Variables
 
+    /**
+     * Encoder cache.
+     */
+    private static final ConcurrentMap<Charset, SynchronizedStack<B2CConverter>> encoders = new ConcurrentHashMap<>();
+
+    // ----------------------------------------------------- Instance Variables
 
     /**
      * The byte buffer.
      */
-    private final ByteChunk bb;
+    private ByteBuffer bb;
 
 
     /**
-     * The chunk buffer.
+     * The char buffer.
      */
-    private CharChunk cb;
+    private CharBuffer cb;
 
 
     /**
@@ -100,18 +102,6 @@ public class InputBuffer extends Reader
     private String enc;
 
 
-    /**
-     * Encoder is set.
-     */
-    private boolean gotEnc = false;
-
-
-    /**
-     * List of encoders.
-     */
-    protected final ConcurrentHashMap<String,B2CConverter> encoders = new ConcurrentHashMap<>();
-
-
     /**
      * Current byte to char converter.
      */
@@ -130,6 +120,12 @@ public class InputBuffer extends Reader
     private int markPos = -1;
 
 
+    /**
+     * Char buffer limit.
+     */
+    private int readLimit;
+
+
     /**
      * Buffer size.
      */
@@ -157,14 +153,11 @@ public class InputBuffer extends Reader
     public InputBuffer(int size) {
 
         this.size = size;
-        bb = new ByteChunk(size);
-        bb.setLimit(size);
-        bb.setByteInputChannel(this);
-        cb = new CharChunk(size);
-        cb.setLimit(size);
-        cb.setOptimizedWrite(false);
-        cb.setCharInputChannel(this);
-        cb.setCharOutputChannel(this);
+        bb = ByteBuffer.allocate(size);
+        clear(bb);
+        cb = CharBuffer.allocate(size);
+        clear(cb);
+        readLimit = size;
 
     }
 
@@ -192,44 +185,34 @@ public class InputBuffer extends Reader
         state = INITIAL_STATE;
 
         // If usage of mark made the buffer too big, reallocate it
-        if (cb.getChars().length > size) {
-            cb = new CharChunk(size);
-            cb.setLimit(size);
-            cb.setOptimizedWrite(false);
-            cb.setCharInputChannel(this);
-            cb.setCharOutputChannel(this);
+        if (cb.capacity() > size) {
+            cb = CharBuffer.allocate(size);
+            clear(cb);
         } else {
-            cb.recycle();
+            clear(cb);
         }
+        readLimit = size;
         markPos = -1;
-        bb.recycle();
+        clear(bb);
         closed = false;
 
         if (conv != null) {
             conv.recycle();
+            encoders.get(conv.getCharset()).push(conv);
+            conv = null;
         }
 
-        gotEnc = false;
         enc = null;
     }
 
 
-    /**
-     * Clear cached encoders (to save memory for Comet requests).
-     */
-    public void clearEncoders() {
-        encoders.clear();
-    }
-
-
     /**
      * Close the input buffer.
      *
      * @throws IOException An underlying IOException occurred
      */
     @Override
-    public void close()
-        throws IOException {
+    public void close() throws IOException {
         closed = true;
     }
 
@@ -237,19 +220,13 @@ public class InputBuffer extends Reader
     public int available() {
         int available = 0;
         if (state == BYTE_STATE) {
-            available = bb.getLength();
+            available = bb.remaining();
         } else if (state == CHAR_STATE) {
-            available = cb.getLength();
+            available = cb.remaining();
         }
         if (available == 0) {
-            // Written this way to avoid use of IS_COMET action where possible
-            boolean readForAvailable = coyoteRequest.getReadListener() != null;
-            if (!readForAvailable) {
-                AtomicBoolean isComet = new AtomicBoolean();
-                coyoteRequest.action(ActionCode.IS_COMET, isComet);
-                readForAvailable = isComet.get();
-            }
-            coyoteRequest.action(ActionCode.AVAILABLE, Boolean.valueOf(readForAvailable));
+            coyoteRequest.action(ActionCode.AVAILABLE,
+                    Boolean.valueOf(coyoteRequest.getReadListener() != null));
             available = (coyoteRequest.getAvailable() > 0) ? 1 : 0;
         }
         return available;
@@ -280,9 +257,9 @@ public class InputBuffer extends Reader
     public boolean isFinished() {
         int available = 0;
         if (state == BYTE_STATE) {
-            available = bb.getLength();
+            available = bb.remaining();
         } else if (state == CHAR_STATE) {
-            available = cb.getLength();
+            available = cb.remaining();
         }
         if (available > 0) {
             return false;
@@ -296,8 +273,6 @@ public class InputBuffer extends Reader
         if (coyoteRequest.getReadListener() == null) {
             throw new IllegalStateException(sm.getString("inputBuffer.requiresNonBlocking"));
         }
-        // Need to check is finished before we check available() as BIO always
-        // returns 1 for isAvailable()
         if (isFinished()) {
             // If this is a non-container thread, need to trigger a read
             // which will eventually lead to a call to onAllDataRead() via a
@@ -326,16 +301,10 @@ public class InputBuffer extends Reader
     /**
      * Reads new bytes in the byte chunk.
      *
-     * @param cbuf Byte buffer to be written to the response
-     * @param off Offset
-     * @param len Length
-     *
      * @throws IOException An underlying IOException occurred
      */
     @Override
-    public int realReadBytes(byte cbuf[], int off, int len)
-            throws IOException {
-
+    public int realReadBytes() throws IOException {
         if (closed) {
             return -1;
         }
@@ -343,119 +312,130 @@ public class InputBuffer extends Reader
             return -1;
         }
 
-        if(state == INITIAL_STATE) {
+        if (state == INITIAL_STATE) {
             state = BYTE_STATE;
         }
 
-        int result = coyoteRequest.doRead(bb);
+        int result = coyoteRequest.doRead(this);
 
         return result;
-
     }
 
 
-    public int readByte()
-        throws IOException {
-
+    public int readByte() throws IOException {
         if (closed) {
             throw new IOException(sm.getString("inputBuffer.streamClosed"));
         }
 
-        return bb.substract();
+        if (checkByteBufferEof()) {
+            return -1;
+        }
+        return bb.get() & 0xFF;
     }
 
 
-    public int read(byte[] b, int off, int len)
-        throws IOException {
-
+    public int read(byte[] b, int off, int len) throws IOException {
         if (closed) {
             throw new IOException(sm.getString("inputBuffer.streamClosed"));
         }
 
-        return bb.substract(b, off, len);
+        if (checkByteBufferEof()) {
+            return -1;
+        }
+        int n = Math.min(len, bb.remaining());
+        bb.get(b, off, n);
+        return n;
+    }
+
+
+    /**
+     * Transfers bytes from the buffer to the specified ByteBuffer. After the
+     * operation the position of the ByteBuffer will be returned to the one
+     * before the operation, the limit will be the position incremented by
+     * the number of the transfered bytes.
+     *
+     * @param to the ByteBuffer into which bytes are to be written.
+     * @return an integer specifying the actual number of bytes read, or -1 if
+     *         the end of the stream is reached
+     * @throws IOException if an input or output exception has occurred
+     */
+    public int read(ByteBuffer to) throws IOException {
+        if (closed) {
+            throw new IOException(sm.getString("inputBuffer.streamClosed"));
+        }
+
+        if (checkByteBufferEof()) {
+            return -1;
+        }
+        int n = Math.min(to.remaining(), bb.remaining());
+        int orgLimit = bb.limit();
+        bb.limit(bb.position() + n);
+        to.put(bb);
+        bb.limit(orgLimit);
+        to.limit(to.position()).position(to.position() - n);
+        return n;
     }
 
 
     // ------------------------------------------------- Chars Handling Methods
 
 
-    /**
-     * Since the converter will use append, it is possible to get chars to
-     * be removed from the buffer for "writing". Since the chars have already
-     * been read before, they are ignored. If a mark was set, then the
-     * mark is lost.
-     */
-    @Override
-    public void realWriteChars(char c[], int off, int len)
-        throws IOException {
-        markPos = -1;
-        cb.setOffset(0);
-        cb.setEnd(0);
-    }
-
-
     public void setEncoding(String s) {
         enc = s;
     }
 
 
-    @Override
-    public int realReadChars(char cbuf[], int off, int len)
-        throws IOException {
-
-        if (!gotEnc) {
-            setConverter();
-        }
+    public int realReadChars() throws IOException {
+        checkConverter();
 
         boolean eof = false;
 
-        if (bb.getLength() <= 0) {
-            int nRead = realReadBytes(bb.getBytes(), 0, bb.getBytes().length);
+        if (bb.remaining() <= 0) {
+            int nRead = realReadBytes();
             if (nRead < 0) {
                 eof = true;
             }
         }
 
         if (markPos == -1) {
-            cb.setOffset(0);
-            cb.setEnd(0);
+            clear(cb);
         } else {
             // Make sure there's enough space in the worst case
-            cb.makeSpace(bb.getLength());
-            if ((cb.getBuffer().length - cb.getEnd()) == 0 && bb.getLength() != 0) {
+            makeSpace(bb.remaining());
+            if ((cb.capacity() - cb.limit()) == 0 && bb.remaining() != 0) {
                 // We went over the limit
-                cb.setOffset(0);
-                cb.setEnd(0);
+                clear(cb);
                 markPos = -1;
             }
         }
 
         state = CHAR_STATE;
-        conv.convert(bb, cb, eof);
+        conv.convert(bb, cb, this, eof);
 
-        if (cb.getLength() == 0 && eof) {
+        if (cb.remaining() == 0 && eof) {
             return -1;
         } else {
-            return cb.getLength();
+            return cb.remaining();
         }
     }
 
 
     @Override
-    public int read()
-        throws IOException {
+    public int read() throws IOException {
 
         if (closed) {
             throw new IOException(sm.getString("inputBuffer.streamClosed"));
         }
 
-        return cb.substract();
+        if (checkCharBufferEof()) {
+            return -1;
+        }
+        return cb.get();
     }
 
 
     @Override
-    public int read(char[] cbuf)
-        throws IOException {
+    public int read(char[] cbuf) throws IOException {
 
         if (closed) {
             throw new IOException(sm.getString("inputBuffer.streamClosed"));
@@ -466,22 +446,23 @@ public class InputBuffer extends Reader
 
 
     @Override
-    public int read(char[] cbuf, int off, int len)
-        throws IOException {
+    public int read(char[] cbuf, int off, int len) throws IOException {
 
         if (closed) {
             throw new IOException(sm.getString("inputBuffer.streamClosed"));
         }
 
-        return cb.substract(cbuf, off, len);
+        if (checkCharBufferEof()) {
+            return -1;
+        }
+        int n = Math.min(len, cb.remaining());
+        cb.get(cbuf, off, n);
+        return n;
     }
 
 
     @Override
-    public long skip(long n)
-        throws IOException {
-
-
+    public long skip(long n) throws IOException {
         if (closed) {
             throw new IOException(sm.getString("inputBuffer.streamClosed"));
         }
@@ -492,34 +473,24 @@ public class InputBuffer extends Reader
 
         long nRead = 0;
         while (nRead < n) {
-            if (cb.getLength() >= n) {
-                cb.setOffset(cb.getStart() + (int) n);
+            if (cb.remaining() >= n) {
+                cb.position(cb.position() + (int) n);
                 nRead = n;
             } else {
-                nRead += cb.getLength();
-                cb.setOffset(cb.getEnd());
-                int toRead = 0;
-                if (cb.getChars().length < (n - nRead)) {
-                    toRead = cb.getChars().length;
-                } else {
-                    toRead = (int) (n - nRead);
-                }
-                int nb = realReadChars(cb.getChars(), 0, toRead);
+                nRead += cb.remaining();
+                cb.position(cb.limit());
+                int nb = realReadChars();
                 if (nb < 0) {
                     break;
                 }
             }
         }
-
         return nRead;
-
     }
 
 
     @Override
-    public boolean ready()
-        throws IOException {
-
+    public boolean ready() throws IOException {
         if (closed) {
             throw new IOException(sm.getString("inputBuffer.streamClosed"));
         }
@@ -537,33 +508,27 @@ public class InputBuffer extends Reader
 
 
     @Override
-    public void mark(int readAheadLimit)
-        throws IOException {
+    public void mark(int readAheadLimit) throws IOException {
 
         if (closed) {
             throw new IOException(sm.getString("inputBuffer.streamClosed"));
         }
 
-        if (cb.getLength() <= 0) {
-            cb.setOffset(0);
-            cb.setEnd(0);
+        if (cb.remaining() <= 0) {
+            clear(cb);
         } else {
-            if ((cb.getBuffer().length > (2 * size))
-                && (cb.getLength()) < (cb.getStart())) {
-                System.arraycopy(cb.getBuffer(), cb.getStart(),
-                                 cb.getBuffer(), 0, cb.getLength());
-                cb.setEnd(cb.getLength());
-                cb.setOffset(0);
+            if ((cb.capacity() > (2 * size)) && (cb.remaining()) < (cb.position())) {
+                cb.compact();
+                cb.flip();
             }
         }
-        cb.setLimit(cb.getStart() + readAheadLimit + size);
-        markPos = cb.getStart();
+        readLimit = cb.position() + readAheadLimit + size;
+        markPos = cb.position();
     }
 
 
     @Override
-    public void reset()
-        throws IOException {
+    public void reset() throws IOException {
 
         if (closed) {
             throw new IOException(sm.getString("inputBuffer.streamClosed"));
@@ -571,65 +536,142 @@ public class InputBuffer extends Reader
 
         if (state == CHAR_STATE) {
             if (markPos < 0) {
-                cb.recycle();
+                clear(cb);
                 markPos = -1;
                 throw new IOException();
             } else {
-                cb.setOffset(markPos);
+                cb.position(markPos);
             }
         } else {
-            bb.recycle();
+            clear(bb);
         }
     }
 
 
-    public void checkConverter()
-        throws IOException {
-
-        if (!gotEnc) {
+    public void checkConverter() throws IOException {
+        if (conv == null) {
             setConverter();
         }
-
     }
 
 
-    protected void setConverter()
-        throws IOException {
-
+    private void setConverter() throws IOException {
         if (coyoteRequest != null) {
             enc = coyoteRequest.getCharacterEncoding();
         }
 
-        gotEnc = true;
         if (enc == null) {
-            enc = DEFAULT_ENCODING;
+            enc = org.apache.coyote.Constants.DEFAULT_CHARACTER_ENCODING;
         }
-        conv = encoders.get(enc);
+
+        Charset charset = B2CConverter.getCharset(enc);
+        SynchronizedStack<B2CConverter> stack = encoders.get(charset);
+        if (stack == null) {
+            stack = new SynchronizedStack<>();
+            encoders.putIfAbsent(charset, stack);
+            stack = encoders.get(charset);
+        }
+        conv = stack.pop();
+
         if (conv == null) {
-            if (SecurityUtil.isPackageProtectionEnabled()){
-                try{
-                    conv = AccessController.doPrivileged(
-                            new PrivilegedExceptionAction<B2CConverter>(){
+            conv = createConverter(charset);
+        }
+    }
 
-                                @Override
-                                public B2CConverter run() throws IOException {
-                                    return new B2CConverter(enc);
-                                }
 
-                            }
-                    );
-                }catch(PrivilegedActionException ex){
-                    Exception e = ex.getException();
-                    if (e instanceof IOException) {
-                        throw (IOException)e;
+    private static B2CConverter createConverter(final Charset charset) throws IOException {
+        if (SecurityUtil.isPackageProtectionEnabled()) {
+            try {
+                return AccessController.doPrivileged(new PrivilegedExceptionAction<B2CConverter>() {
+
+                    @Override
+                    public B2CConverter run() throws IOException {
+                        return new B2CConverter(charset);
                     }
+                });
+            } catch (PrivilegedActionException ex) {
+                Exception e = ex.getException();
+                if (e instanceof IOException) {
+                    throw (IOException) e;
+                } else {
+                    throw new IOException(e);
                 }
-            } else {
-                conv = new B2CConverter(enc);
             }
-            encoders.put(enc, conv);
+        } else {
+            return new B2CConverter(charset);
         }
 
     }
 
+
+    @Override
+    public void setByteBuffer(ByteBuffer buffer) {
+        bb = buffer;
+    }
+
+
+    @Override
+    public ByteBuffer getByteBuffer() {
+        return bb;
+    }
+
+
+    @Override
+    public void expand(int size) {
+        // no-op
+    }
+
+
+    private boolean checkByteBufferEof() throws IOException {
+        if (bb.remaining() == 0) {
+            int n = realReadBytes();
+            if (n < 0) {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    private boolean checkCharBufferEof() throws IOException {
+        if (cb.remaining() == 0) {
+            int n = realReadChars();
+            if (n < 0) {
+                return true;
+            }
+        }
+        return false;
+    }
+
+    private void clear(Buffer buffer) {
+        buffer.rewind().limit(0);
+    }
+
+    private void makeSpace(int count) {
+        int desiredSize = cb.limit() + count;
+        if(desiredSize > readLimit) {
+            desiredSize = readLimit;
+        }
+
+        if(desiredSize <= cb.capacity()) {
+            return;
+        }
+
+        int newSize = 2 * cb.capacity();
+        if(desiredSize >= newSize) {
+            newSize= 2 * cb.capacity() + count;
+        }
+
+        if (newSize > readLimit) {
+            newSize = readLimit;
+        }
+
+        CharBuffer tmp = CharBuffer.allocate(newSize);
+        int oldPosition = cb.position();
+        cb.position(0);
+        tmp.put(cb);
+        tmp.flip();
+        tmp.position(oldPosition);
+        cb = tmp;
+        tmp = null;
+    }
 }
