@@ -18,10 +18,15 @@ package org.apache.catalina.connector;
 
 import java.io.IOException;
 import java.io.Writer;
+import java.nio.Buffer;
+import java.nio.ByteBuffer;
+import java.nio.CharBuffer;
+import java.nio.charset.Charset;
 import java.security.AccessController;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
-import java.util.concurrent.ConcurrentHashMap;
+import java.util.HashMap;
+import java.util.Map;
 
 import javax.servlet.WriteListener;
 import javax.servlet.http.HttpServletResponse;
@@ -29,9 +34,8 @@ import javax.servlet.http.HttpServletResponse;
 import org.apache.catalina.Globals;
 import org.apache.coyote.ActionCode;
 import org.apache.coyote.Response;
-import org.apache.tomcat.util.buf.ByteChunk;
+import org.apache.tomcat.util.buf.B2CConverter;
 import org.apache.tomcat.util.buf.C2BConverter;
-import org.apache.tomcat.util.buf.CharChunk;
 import org.apache.tomcat.util.res.StringManager;
 
 /**
@@ -42,17 +46,16 @@ import org.apache.tomcat.util.res.StringManager;
  * @author Costin Manolache
  * @author Remy Maucherat
  */
-public class OutputBuffer extends Writer
-    implements ByteChunk.ByteOutputChannel, CharChunk.CharOutputChannel {
+public class OutputBuffer extends Writer {
 
-    private static final StringManager sm =
-            StringManager.getManager(Constants.Package);
+    private static final StringManager sm = StringManager.getManager(OutputBuffer.class);
 
-    // -------------------------------------------------------------- Constants
+    public static final int DEFAULT_BUFFER_SIZE = 8 * 1024;
 
-    public static final String DEFAULT_ENCODING =
-        org.apache.coyote.Constants.DEFAULT_CHARACTER_ENCODING;
-    public static final int DEFAULT_BUFFER_SIZE = 8*1024;
+    /**
+     * Encoder cache.
+     */
+    private final Map<Charset, C2BConverter> encoders = new HashMap<>();
 
 
     // ----------------------------------------------------- Instance Variables
@@ -60,13 +63,13 @@ public class OutputBuffer extends Writer
     /**
      * The byte buffer.
      */
-    private final ByteChunk bb;
+    private ByteBuffer bb;
 
 
     /**
-     * The chunk buffer.
+     * The char buffer.
      */
-    private final CharChunk cb;
+    private final CharBuffer cb;
 
 
     /**
@@ -99,36 +102,12 @@ public class OutputBuffer extends Writer
     private boolean doFlush = false;
 
 
-    /**
-     * Byte chunk used to output bytes.
-     */
-    private final ByteChunk outputChunk = new ByteChunk();
-
-
-    /**
-     * Char chunk used to output chars.
-     */
-    private CharChunk outputCharChunk = new CharChunk();
-
-
     /**
      * Encoding to use.
      */
     private String enc;
 
 
-    /**
-     * Encoder is set.
-     */
-    private boolean gotEnc = false;
-
-
-    /**
-     * List of encoders.
-     */
-    protected final ConcurrentHashMap<String, C2BConverter> encoders = new ConcurrentHashMap<>();
-
-
     /**
      * Current char to byte converter.
      */
@@ -167,13 +146,10 @@ public class OutputBuffer extends Writer
      */
     public OutputBuffer(int size) {
 
-        bb = new ByteChunk(size);
-        bb.setLimit(size);
-        bb.setByteOutputChannel(this);
-        cb = new CharChunk(size);
-        cb.setLimit(size);
-        cb.setOptimizedWrite(false);
-        cb.setCharOutputChannel(this);
+        bb = ByteBuffer.allocate(size);
+        clear(bb);
+        cb = CharBuffer.allocate(size);
+        clear(cb);
 
     }
 
@@ -232,30 +208,21 @@ public class OutputBuffer extends Writer
         bytesWritten = 0;
         charsWritten = 0;
 
-        bb.recycle();
-        cb.recycle();
-        outputCharChunk.setChars(null, 0, 0);
+        clear(bb);
+        clear(cb);
         closed = false;
         suspended = false;
         doFlush = false;
 
-        if (conv!= null) {
+        if (conv != null) {
             conv.recycle();
+            conv = null;
         }
 
-        gotEnc = false;
         enc = null;
     }
 
 
-    /**
-     * Clear cached encoders (to save memory for Comet requests).
-     */
-    public void clearEncoders() {
-        encoders.clear();
-    }
-
-
     /**
      * Close the output buffer. This tries to calculate the response size if
      * the response has not been committed yet.
@@ -263,8 +230,7 @@ public class OutputBuffer extends Writer
      * @throws IOException An underlying IOException occurred
      */
     @Override
-    public void close()
-        throws IOException {
+    public void close() throws IOException {
 
         if (closed) {
             return;
@@ -275,21 +241,23 @@ public class OutputBuffer extends Writer
 
         // If there are chars, flush all of them to the byte buffer now as bytes are used to
         // calculate the content-length (if everything fits into the byte buffer, of course).
-        if (cb.getLength() > 0) {
-            cb.flushBuffer();
+        if (cb.remaining() > 0) {
+            flushCharBuffer();
         }
 
-        if ((!coyoteResponse.isCommitted())
-            && (coyoteResponse.getContentLengthLong() == -1)) {
+        if ((!coyoteResponse.isCommitted()) && (coyoteResponse.getContentLengthLong() == -1)
+                && !coyoteResponse.getRequest().method().equals("HEAD")) {
             // If this didn't cause a commit of the response, the final content
-            // length can be calculated
+            // length can be calculated. Only do this if this is not a HEAD
+            // request since in that case no body should have been written and
+            // setting a value of zero here will result in an explicit content
+            // length of zero being set on the response.
             if (!coyoteResponse.isCommitted()) {
-                coyoteResponse.setContentLength(bb.getLength());
+                coyoteResponse.setContentLength(bb.remaining());
             }
         }
 
-        if (coyoteResponse.getStatus() ==
-                HttpServletResponse.SC_SWITCHING_PROTOCOLS) {
+        if (coyoteResponse.getStatus() == HttpServletResponse.SC_SWITCHING_PROTOCOLS) {
             doFlush(true);
         } else {
             doFlush(false);
@@ -299,8 +267,7 @@ public class OutputBuffer extends Writer
         // The request should have been completely read by the time the response
         // is closed. Further reads of the input a) are pointless and b) really
         // confuse AJP (bug 50189) so close the input buffer to prevent them.
-        Request req = (Request) coyoteResponse.getRequest().getNote(
-                CoyoteAdapter.ADAPTER_NOTES);
+        Request req = (Request) coyoteResponse.getRequest().getNote(CoyoteAdapter.ADAPTER_NOTES);
         req.inputBuffer.close();
 
         coyoteResponse.action(ActionCode.CLOSE, null);
@@ -321,6 +288,7 @@ public class OutputBuffer extends Writer
     /**
      * Flush bytes or chars contained in the buffer.
      *
+     * @param realFlush <code>true</code> if this should also cause a real network flush
      * @throws IOException An underlying IOException occurred
      */
     protected void doFlush(boolean realFlush) throws IOException {
@@ -335,11 +303,11 @@ public class OutputBuffer extends Writer
                 coyoteResponse.sendHeaders();
                 initial = false;
             }
-            if (cb.getLength() > 0) {
-                cb.flushBuffer();
+            if (cb.remaining() > 0) {
+                flushCharBuffer();
             }
-            if (bb.getLength() > 0) {
-                bb.flushBuffer();
+            if (bb.remaining() > 0) {
+                flushByteBuffer();
             }
         } finally {
             doFlush = false;
@@ -363,15 +331,11 @@ public class OutputBuffer extends Writer
      * Sends the buffer data to the client output, checking the
      * state of Response and calling the right interceptors.
      *
-     * @param buf Byte buffer to be written to the response
-     * @param off Offset
-     * @param cnt Length
+     * @param buf the ByteBuffer to be written to the response
      *
      * @throws IOException An underlying IOException occurred
      */
-    @Override
-    public void realWriteBytes(byte buf[], int off, int cnt)
-            throws IOException {
+    public void realWriteBytes(ByteBuffer buf) throws IOException {
 
         if (closed) {
             return;
@@ -381,14 +345,13 @@ public class OutputBuffer extends Writer
         }
 
         // If we really have something to write
-        if (cnt > 0) {
+        if (buf.remaining() > 0) {
             // real write to the adapter
-            outputChunk.setBytes(buf, off, cnt);
             try {
-                coyoteResponse.doWrite(outputChunk);
+                coyoteResponse.doWrite(buf);
             } catch (IOException e) {
                 // An IOException on a write is almost always due to
-                // the remote client aborting the request.  Wrap this
+                // the remote client aborting the request. Wrap this
                 // so that it can be handled better by the error dispatcher.
                 throw new ClientAbortException(e);
             }
@@ -408,33 +371,64 @@ public class OutputBuffer extends Writer
     }
 
 
-    private void writeBytes(byte b[], int off, int len)
-        throws IOException {
-
-        if (closed) {
-            return;
-        }
-
-        bb.append(b, off, len);
-        bytesWritten += len;
-
-        // if called from within flush(), then immediately flush
-        // remaining bytes
-        if (doFlush) {
-            bb.flushBuffer();
-        }
-
-    }
-
-
-    public void writeByte(int b)
-        throws IOException {
+    public void write(ByteBuffer from) throws IOException {
 
         if (suspended) {
             return;
         }
 
-        bb.append((byte) b);
+        writeBytes(from);
+
+    }
+
+
+    private void writeBytes(byte b[], int off, int len) throws IOException {
+
+        if (closed) {
+            return;
+        }
+
+        append(b, off, len);
+        bytesWritten += len;
+
+        // if called from within flush(), then immediately flush
+        // remaining bytes
+        if (doFlush) {
+            flushByteBuffer();
+        }
+
+    }
+
+
+    private void writeBytes(ByteBuffer from) throws IOException {
+
+        if (closed) {
+            return;
+        }
+
+        append(from);
+        bytesWritten += from.remaining();
+
+        // if called from within flush(), then immediately flush
+        // remaining bytes
+        if (doFlush) {
+            flushByteBuffer();
+        }
+
+    }
+
+
+    public void writeByte(int b) throws IOException {
+
+        if (suspended) {
+            return;
+        }
+
+        if (isFull(bb)) {
+            flushByteBuffer();
+        }
+
+        transfer((byte) b, bb);
         bytesWritten++;
 
     }
@@ -446,52 +440,44 @@ public class OutputBuffer extends Writer
     /**
      * Convert the chars to bytes, then send the data to the client.
      *
-     * @param buf Char buffer to be written to the response
-     * @param off Offset
-     * @param len Length
+     * @param from Char buffer to be written to the response
      *
      * @throws IOException An underlying IOException occurred
      */
-    @Override
-    public void realWriteChars(char buf[], int off, int len)
-        throws IOException {
+    public void realWriteChars(CharBuffer from) throws IOException {
 
-        outputCharChunk.setChars(buf, off, len);
-        while (outputCharChunk.getLength() > 0) {
-            conv.convert(outputCharChunk, bb);
-            if (bb.getLength() == 0) {
+        while (from.remaining() > 0) {
+            conv.convert(from, bb);
+            if (bb.remaining() == 0) {
                 // Break out of the loop if more chars are needed to produce any output
                 break;
             }
-            if (outputCharChunk.getLength() > 0) {
-                if (bb.getBuffer().length == bb.getEnd() && bb.getLength() < bb.getLimit()) {
-                    // Need to expand output buffer
-                    bb.makeSpace(outputCharChunk.getLength());
-                } else {
-                    bb.flushBuffer();
-                }
+            if (from.remaining() > 0) {
+                flushByteBuffer();
             }
         }
 
     }
 
     @Override
-    public void write(int c)
-        throws IOException {
+    public void write(int c) throws IOException {
 
         if (suspended) {
             return;
         }
 
-        cb.append((char) c);
+        if (isFull(cb)) {
+            flushCharBuffer();
+        }
+
+        transfer((char) c, cb);
         charsWritten++;
 
     }
 
 
     @Override
-    public void write(char c[])
-        throws IOException {
+    public void write(char c[]) throws IOException {
 
         if (suspended) {
             return;
@@ -503,14 +489,13 @@ public class OutputBuffer extends Writer
 
 
     @Override
-    public void write(char c[], int off, int len)
-        throws IOException {
+    public void write(char c[], int off, int len) throws IOException {
 
         if (suspended) {
             return;
         }
 
-        cb.append(c, off, len);
+        append(c, off, len);
         charsWritten += len;
 
     }
@@ -520,8 +505,7 @@ public class OutputBuffer extends Writer
      * Append a string to the buffer
      */
     @Override
-    public void write(String s, int off, int len)
-        throws IOException {
+    public void write(String s, int off, int len) throws IOException {
 
         if (suspended) {
             return;
@@ -530,14 +514,23 @@ public class OutputBuffer extends Writer
         if (s == null) {
             throw new NullPointerException(sm.getString("outputBuffer.writeNull"));
         }
-        cb.append(s, off, len);
+
+        int sOff = off;
+        int sEnd = off + len;
+        while (sOff < sEnd) {
+            int n = transfer(s, sOff, sEnd - sOff, cb);
+            sOff += n;
+            if (isFull(cb)) {
+                flushCharBuffer();
+            }
+        }
+
         charsWritten += len;
     }
 
 
     @Override
-    public void write(String s)
-        throws IOException {
+    public void write(String s) throws IOException {
 
         if (suspended) {
             return;
@@ -546,8 +539,7 @@ public class OutputBuffer extends Writer
         if (s == null) {
             s = "null";
         }
-        cb.append(s);
-        charsWritten += s.length();
+        write(s, 0, s.length());
     }
 
 
@@ -556,60 +548,81 @@ public class OutputBuffer extends Writer
     }
 
 
-    public void checkConverter()
-        throws IOException {
-
-        if (!gotEnc) {
+    public void checkConverter() throws IOException {
+        if (conv == null) {
             setConverter();
         }
-
     }
 
 
-    protected void setConverter()
-        throws IOException {
+    private void setConverter() throws IOException {
 
         if (coyoteResponse != null) {
             enc = coyoteResponse.getCharacterEncoding();
         }
 
-        gotEnc = true;
         if (enc == null) {
-            enc = DEFAULT_ENCODING;
+            enc = org.apache.coyote.Constants.DEFAULT_CHARACTER_ENCODING;
         }
-        conv = encoders.get(enc);
+
+        final Charset charset = getCharset(enc);
+        conv = encoders.get(charset);
+
         if (conv == null) {
-            if (Globals.IS_SECURITY_ENABLED){
-                try{
-                    conv = AccessController.doPrivileged(
-                            new PrivilegedExceptionAction<C2BConverter>(){
+            conv = createConverter(charset);
+            encoders.put(charset, conv);
+        }
+    }
 
-                                @Override
-                                public C2BConverter run() throws IOException{
-                                    return new C2BConverter(enc);
-                                }
 
-                            }
-                    );
-                }catch(PrivilegedActionException ex){
-                    Exception e = ex.getException();
-                    if (e instanceof IOException) {
-                        throw (IOException)e;
+    private static Charset getCharset(final String encoding) throws IOException {
+        if (Globals.IS_SECURITY_ENABLED) {
+            try {
+                return AccessController.doPrivileged(new PrivilegedExceptionAction<Charset>() {
+                    @Override
+                    public Charset run() throws IOException {
+                        return B2CConverter.getCharset(encoding);
                     }
+                });
+            } catch (PrivilegedActionException ex) {
+                Exception e = ex.getException();
+                if (e instanceof IOException) {
+                    throw (IOException) e;
+                } else {
+                    throw new IOException(ex);
                 }
-            } else {
-                conv = new C2BConverter(enc);
             }
+        } else {
+            return B2CConverter.getCharset(encoding);
+        }
+    }
 
-            encoders.put(enc, conv);
 
+    private static C2BConverter createConverter(final Charset charset) throws IOException {
+        if (Globals.IS_SECURITY_ENABLED) {
+            try {
+                return AccessController.doPrivileged(new PrivilegedExceptionAction<C2BConverter>() {
+                    @Override
+                    public C2BConverter run() throws IOException {
+                        return new C2BConverter(charset);
+                    }
+                });
+            } catch (PrivilegedActionException ex) {
+                Exception e = ex.getException();
+                if (e instanceof IOException) {
+                    throw (IOException) e;
+                } else {
+                    throw new IOException(ex);
+                }
+            }
+        } else {
+            return new C2BConverter(charset);
         }
     }
 
 
     // --------------------  BufferedOutputStream compatibility
 
-
     public long getContentWritten() {
         return bytesWritten + charsWritten;
     }
@@ -626,8 +639,9 @@ public class OutputBuffer extends Writer
 
 
     public void setBufferSize(int size) {
-        if (size > bb.getLimit()) {// ??????
-            bb.setLimit(size);
+        if (size > bb.capacity()) {
+            bb = ByteBuffer.allocate(size);
+            clear(bb);
         }
     }
 
@@ -637,12 +651,15 @@ public class OutputBuffer extends Writer
     }
 
     public void reset(boolean resetWriterStreamFlags) {
-        bb.recycle();
-        cb.recycle();
+        clear(bb);
+        clear(cb);
         bytesWritten = 0;
         charsWritten = 0;
         if (resetWriterStreamFlags) {
-            gotEnc = false;
+            if (conv != null) {
+                conv.recycle();
+            }
+            conv = null;
             enc = null;
         }
         initial = true;
@@ -650,7 +667,7 @@ public class OutputBuffer extends Writer
 
 
     public int getBufferSize() {
-        return bb.getLimit();
+        return bb.capacity();
     }
 
 
@@ -676,4 +693,197 @@ public class OutputBuffer extends Writer
     public void checkRegisterForWrite() {
         coyoteResponse.checkRegisterForWrite();
     }
+
+    /**
+     * Add data to the buffer.
+     *
+     * @param src Bytes array
+     * @param off Offset
+     * @param len Length
+     * @throws IOException Writing overflow data to the output channel failed
+     */
+    public void append(byte src[], int off, int len) throws IOException {
+        if (bb.remaining() == 0) {
+            appendByteArray(src, off, len);
+        } else {
+            int n = transfer(src, off, len, bb);
+            len = len - n;
+            off = off + n;
+            if (isFull(bb)) {
+                flushByteBuffer();
+                appendByteArray(src, off, len);
+            }
+        }
+    }
+
+    /**
+     * Add data to the buffer.
+     * @param src Char array
+     * @param off Offset
+     * @param len Length
+     * @throws IOException Writing overflow data to the output channel failed
+     */
+    public void append(char src[], int off, int len) throws IOException {
+        // if we have limit and we're below
+        if(len <= cb.capacity() - cb.limit()) {
+            transfer(src, off, len, cb);
+            return;
+        }
+
+        // Optimization:
+        // If len-avail < length ( i.e. after we fill the buffer with
+        // what we can, the remaining will fit in the buffer ) we'll just
+        // copy the first part, flush, then copy the second part - 1 write
+        // and still have some space for more. We'll still have 2 writes, but
+        // we write more on the first.
+        if(len + cb.limit() < 2 * cb.capacity()) {
+            /* If the request length exceeds the size of the output buffer,
+               flush the output buffer and then write the data directly.
+               We can't avoid 2 writes, but we can write more on the second
+            */
+            int n = transfer(src, off, len, cb);
+
+            flushCharBuffer();
+
+            transfer(src, off + n, len - n, cb);
+        } else {
+            // long write - flush the buffer and write the rest
+            // directly from source
+            flushCharBuffer();
+
+            realWriteChars(CharBuffer.wrap(src, off, len));
+        }
+    }
+
+
+    public void append(ByteBuffer from) throws IOException {
+        if (bb.remaining() == 0) {
+            appendByteBuffer(from);
+        } else {
+            transfer(from, bb);
+            if (isFull(bb)) {
+                flushByteBuffer();
+                appendByteBuffer(from);
+            }
+        }
+    }
+
+    private void appendByteArray(byte src[], int off, int len) throws IOException {
+        if (len == 0) {
+            return;
+        }
+
+        int limit = bb.capacity();
+        while (len >= limit) {
+            realWriteBytes(ByteBuffer.wrap(src, off, limit));
+            len = len - limit;
+            off = off + limit;
+        }
+
+        if (len > 0) {
+            transfer(src, off, len, bb);
+        }
+    }
+
+    private void appendByteBuffer(ByteBuffer from) throws IOException {
+        if (from.remaining() == 0) {
+            return;
+        }
+
+        int limit = bb.capacity();
+        int fromLimit = from.limit();
+        while (from.remaining() >= limit) {
+            from.limit(from.position() + limit);
+            realWriteBytes(from.slice());
+            from.position(from.limit());
+            from.limit(fromLimit);
+        }
+
+        if (from.remaining() > 0) {
+            transfer(from, bb);
+        }
+    }
+
+    private void flushByteBuffer() throws IOException {
+        realWriteBytes(bb.slice());
+        clear(bb);
+    }
+
+    private void flushCharBuffer() throws IOException {
+        realWriteChars(cb.slice());
+        clear(cb);
+    }
+
+    private void transfer(byte b, ByteBuffer to) {
+        toWriteMode(to);
+        to.put(b);
+        toReadMode(to);
+    }
+
+    private void transfer(char b, CharBuffer to) {
+        toWriteMode(to);
+        to.put(b);
+        toReadMode(to);
+    }
+
+    private int transfer(byte[] buf, int off, int len, ByteBuffer to) {
+        toWriteMode(to);
+        int max = Math.min(len, to.remaining());
+        if (max > 0) {
+            to.put(buf, off, max);
+        }
+        toReadMode(to);
+        return max;
+    }
+
+    private int transfer(char[] buf, int off, int len, CharBuffer to) {
+        toWriteMode(to);
+        int max = Math.min(len, to.remaining());
+        if (max > 0) {
+            to.put(buf, off, max);
+        }
+        toReadMode(to);
+        return max;
+    }
+
+    private int transfer(String s, int off, int len, CharBuffer to) {
+        toWriteMode(to);
+        int max = Math.min(len, to.remaining());
+        if (max > 0) {
+            to.put(s, off, off + max);
+        }
+        toReadMode(to);
+        return max;
+    }
+
+    private void transfer(ByteBuffer from, ByteBuffer to) {
+        toWriteMode(to);
+        int max = Math.min(from.remaining(), to.remaining());
+        if (max > 0) {
+            int fromLimit = from.limit();
+            from.limit(from.position() + max);
+            to.put(from);
+            from.limit(fromLimit);
+        }
+        toReadMode(to);
+    }
+
+    private void clear(Buffer buffer) {
+        buffer.rewind().limit(0);
+    }
+
+    private boolean isFull(Buffer buffer) {
+        return buffer.limit() == buffer.capacity();
+    }
+
+    private void toReadMode(Buffer buffer) {
+        buffer.limit(buffer.position())
+              .reset();
+    }
+
+    private void toWriteMode(Buffer buffer) {
+        buffer.mark()
+              .position(buffer.limit())
+              .limit(buffer.capacity());
+    }
 }
