@@ -39,6 +39,7 @@ public final class StandardWrapperFacade
 
     /**
      * Create a new facade around a StandardWrapper.
+     * @param config the associated wrapper
      */
     public StandardWrapperFacade(StandardWrapper config) {
 
@@ -76,8 +77,9 @@ public final class StandardWrapperFacade
     public ServletContext getServletContext() {
         if (context == null) {
             context = config.getServletContext();
-            if ((context != null) && (context instanceof ApplicationContext))
+            if (context instanceof ApplicationContext) {
                 context = ((ApplicationContext) context).getFacade();
+            }
         }
         return (context);
     }
