@@ -304,6 +304,7 @@ public final class HTMLManagerServlet extends ManagerServlet {
      * @param config URL of the context configuration file to be deployed
      * @param cn Name of the application to be deployed
      * @param war URL of the web application archive to be deployed
+     * @param smClient internationalized strings
      * @return message String
      */
     protected String deployInternal(String config, ContextName cn, String war,
@@ -324,6 +325,8 @@ public final class HTMLManagerServlet extends ManagerServlet {
      * @param request The request
      * @param response The response
      * @param message a message to display
+     * @param smClient internationalized strings
+     * @throws IOException an IO error occurred
      */
     protected void list(HttpServletRequest request,
                      HttpServletResponse response,
@@ -427,10 +430,10 @@ public final class HTMLManagerServlet extends ManagerServlet {
 
                 StringBuilder tmp = new StringBuilder();
                 tmp.append("path=");
-                tmp.append(URL_ENCODER.encode(displayPath));
+                tmp.append(URL_ENCODER.encode(displayPath, "UTF-8"));
                 if (ctxt.getWebappVersion().length() > 0) {
                     tmp.append("&version=");
-                    tmp.append(URL_ENCODER.encode(ctxt.getWebappVersion()));
+                    tmp.append(URL_ENCODER.encode(ctxt.getWebappVersion(), "UTF-8"));
                 }
                 String pathVersion = tmp.toString();
 
@@ -442,7 +445,7 @@ public final class HTMLManagerServlet extends ManagerServlet {
                 }
 
                 args = new Object[7];
-                args[0] = "<a href=\"" + URL_ENCODER.encode(contextPath + "/")
+                args[0] = "<a href=\"" + URL_ENCODER.encode(contextPath + "/", "UTF-8")
                         + "\">" + RequestUtil.filter(displayPath) + "</a>";
                 if ("".equals(ctxt.getWebappVersion())) {
                     args[1] = noVersion;
@@ -559,8 +562,8 @@ public final class HTMLManagerServlet extends ManagerServlet {
         args[4] = smClient.getString("htmlManagerServlet.serverOSName");
         args[5] = smClient.getString("htmlManagerServlet.serverOSVersion");
         args[6] = smClient.getString("htmlManagerServlet.serverOSArch");
-        args[7] = sm.getString("htmlManagerServlet.serverHostname");
-        args[8] = sm.getString("htmlManagerServlet.serverIPAddress");
+        args[7] = smClient.getString("htmlManagerServlet.serverHostname");
+        args[8] = smClient.getString("htmlManagerServlet.serverIPAddress");
         writer.print(MessageFormat.format
                      (Constants.SERVER_HEADER_SECTION, args));
 
@@ -756,8 +759,9 @@ public final class HTMLManagerServlet extends ManagerServlet {
      * Extract the expiration request parameter
      *
      * @param cn Name of the application from which to expire sessions
-     * @param req
-     * @param smClient  StringManager for the client's locale
+     * @param req The Servlet request
+     * @param smClient StringManager for the client's locale
+     * @return message string
      */
     protected String expireSessions(ContextName cn, HttpServletRequest req,
             StringManager smClient) {
@@ -774,12 +778,14 @@ public final class HTMLManagerServlet extends ManagerServlet {
     }
 
     /**
+     * Handle session operations.
      *
-     * @param req
-     * @param resp
-     * @param smClient  StringManager for the client's locale
-     * @throws ServletException
-     * @throws IOException
+     * @param cn Name of the application for the sessions operation
+     * @param req The Servlet request
+     * @param resp The Servlet response
+     * @param smClient StringManager for the client's locale
+     * @throws ServletException Propagated Servlet error
+     * @throws IOException An IO error occurred
      */
     protected void doSessions(ContextName cn, HttpServletRequest req,
             HttpServletResponse resp, StringManager smClient)
@@ -864,13 +870,13 @@ public final class HTMLManagerServlet extends ManagerServlet {
     }
 
     /**
-     *
+     * List session.
      * @param cn Name of the application for which the sessions will be listed
-     * @param req
-     * @param resp
-     * @param smClient  StringManager for the client's locale
-     * @throws ServletException
-     * @throws IOException
+     * @param req The Servlet request
+     * @param resp The Servlet response
+     * @param smClient StringManager for the client's locale
+     * @throws ServletException Propagated Servlet error
+     * @throws IOException An IO error occurred
      */
     protected void displaySessionsListPage(ContextName cn,
             HttpServletRequest req, HttpServletResponse resp,
@@ -913,12 +919,15 @@ public final class HTMLManagerServlet extends ManagerServlet {
     }
 
     /**
+     * Display session details.
      *
-     * @param req
-     * @param resp
-     * @param smClient  StringManager for the client's locale
-     * @throws ServletException
-     * @throws IOException
+     * @param req The Servlet request
+     * @param resp The Servlet response
+     * @param cn Name of the application for which the sessions will be listed
+     * @param sessionId the session id
+     * @param smClient StringManager for the client's locale
+     * @throws ServletException Propagated Servlet error
+     * @throws IOException An IO error occurred
      */
     protected void displaySessionDetailPage(HttpServletRequest req,
             HttpServletResponse resp, ContextName cn, String sessionId,
@@ -935,16 +944,16 @@ public final class HTMLManagerServlet extends ManagerServlet {
     }
 
     /**
-     * Invalidate HttpSessions
+     * Invalidate specified sessions.
+     *
      * @param cn Name of the application for which sessions are to be
      *           invalidated
-     * @param sessionIds
+     * @param sessionIds the session ids of the sessions
      * @param smClient  StringManager for the client's locale
      * @return number of invalidated sessions
-     * @throws IOException
      */
     protected int invalidateSessions(ContextName cn, String[] sessionIds,
-            StringManager smClient) throws IOException {
+            StringManager smClient) {
         if (null == sessionIds) {
             return 0;
         }
@@ -979,14 +988,13 @@ public final class HTMLManagerServlet extends ManagerServlet {
      * Removes an attribute from an HttpSession
      * @param cn Name of the application hosting the session from which the
      *           attribute is to be removed
-     * @param sessionId
-     * @param attributeName
+     * @param sessionId the session id
+     * @param attributeName the attribute name
      * @param smClient  StringManager for the client's locale
      * @return true if there was an attribute removed, false otherwise
-     * @throws IOException
      */
     protected boolean removeSessionAttribute(ContextName cn, String sessionId,
-            String attributeName, StringManager smClient) throws IOException {
+            String attributeName, StringManager smClient) {
         HttpSession session =
             getSessionForNameAndId(cn, sessionId, smClient).getSession();
         if (null == session) {
