@@ -18,18 +18,21 @@
 package org.apache.coyote.http11.filters;
 
 import java.io.IOException;
+import java.nio.BufferOverflowException;
+import java.nio.ByteBuffer;
 import java.nio.charset.StandardCharsets;
 
 import org.apache.coyote.InputBuffer;
 import org.apache.coyote.Request;
 import org.apache.coyote.http11.InputFilter;
 import org.apache.tomcat.util.buf.ByteChunk;
+import org.apache.tomcat.util.net.ApplicationBufferHandler;
 
 /**
  * Input filter responsible for reading and buffering the request body, so that
  * it does not interfere with client SSL handshake messages.
  */
-public class BufferedInputFilter implements InputFilter {
+public class BufferedInputFilter implements InputFilter, ApplicationBufferHandler {
 
     // -------------------------------------------------------------- Constants
 
@@ -39,8 +42,8 @@ public class BufferedInputFilter implements InputFilter {
 
     // ----------------------------------------------------- Instance Variables
 
-    private ByteChunk buffered = null;
-    private final ByteChunk tempRead = new ByteChunk(1024);
+    private ByteBuffer buffered;
+    private ByteBuffer tempRead;
     private InputBuffer buffer;
     private boolean hasRead = false;
 
@@ -64,8 +67,8 @@ public class BufferedInputFilter implements InputFilter {
      */
     public void setLimit(int limit) {
         if (buffered == null) {
-            buffered = new ByteChunk(4048);
-            buffered.setLimit(limit);
+            buffered = ByteBuffer.allocate(limit);
+            buffered.flip();
         }
     }
 
@@ -80,11 +83,13 @@ public class BufferedInputFilter implements InputFilter {
     public void setRequest(Request request) {
         // save off the Request body
         try {
-            while (buffer.doRead(tempRead, request) >= 0) {
-                buffered.append(tempRead);
-                tempRead.recycle();
+            while (buffer.doRead(this) >= 0) {
+                buffered.mark().position(buffered.limit()).limit(buffered.capacity());
+                buffered.put(tempRead);
+                buffered.limit(buffered.position()).reset();
+                tempRead = null;
             }
-        } catch(IOException ioe) {
+        } catch(IOException | BufferOverflowException ioe) {
             // No need for i18n - this isn't going to get logged anywhere
             throw new IllegalStateException(
                     "Request body too large for buffer");
@@ -93,19 +98,36 @@ public class BufferedInputFilter implements InputFilter {
 
     /**
      * Fills the given ByteChunk with the buffered request body.
+     *
+     * @deprecated Unused. Will be removed in Tomcat 9. Use
+     *             {@link #doRead(ApplicationBufferHandler)}
      */
     @Override
-    public int doRead(ByteChunk chunk, Request request) throws IOException {
-        if (hasRead || buffered.getLength() <= 0) {
+    public int doRead(ByteChunk chunk) throws IOException {
+        if (isFinished()) {
             return -1;
         }
 
-        chunk.setBytes(buffered.getBytes(), buffered.getStart(),
-                buffered.getLength());
+        chunk.setBytes(buffered.array(), buffered.arrayOffset() + buffered.position(),
+                buffered.remaining());
         hasRead = true;
         return chunk.getLength();
     }
 
+    /**
+     * Fills the given ByteBuffer with the buffered request body.
+     */
+    @Override
+    public int doRead(ApplicationBufferHandler handler) throws IOException {
+        if (isFinished()) {
+            return -1;
+        }
+
+        handler.setByteBuffer(buffered);
+        hasRead = true;
+        return buffered.remaining();
+    }
+
     @Override
     public void setBuffer(InputBuffer buffer) {
         this.buffer = buffer;
@@ -114,13 +136,12 @@ public class BufferedInputFilter implements InputFilter {
     @Override
     public void recycle() {
         if (buffered != null) {
-            if (buffered.getBuffer().length > 65536) {
+            if (buffered.capacity() > 65536) {
                 buffered = null;
             } else {
-                buffered.recycle();
+                buffered.position(0).limit(0);
             }
         }
-        tempRead.recycle();
         hasRead = false;
         buffer = null;
     }
@@ -137,12 +158,30 @@ public class BufferedInputFilter implements InputFilter {
 
     @Override
     public int available() {
-        return buffered.getLength();
+        return buffered.remaining();
     }
 
 
     @Override
     public boolean isFinished() {
-        return hasRead || buffered.getLength() <= 0;
+        return hasRead || buffered.remaining() <= 0;
+    }
+
+
+    @Override
+    public void setByteBuffer(ByteBuffer buffer) {
+        tempRead = buffer;
+    }
+
+
+    @Override
+    public ByteBuffer getByteBuffer() {
+        return tempRead;
+    }
+
+
+    @Override
+    public void expand(int size) {
+        // no-op
     }
 }
