@@ -20,48 +20,41 @@ import java.io.EOFException;
 import java.io.IOException;
 import java.net.SocketTimeoutException;
 import java.nio.ByteBuffer;
-import java.util.Queue;
-import java.util.concurrent.ConcurrentLinkedQueue;
-import java.util.concurrent.ExecutorService;
+import java.util.concurrent.Executor;
 
-import javax.servlet.ServletOutputStream;
 import javax.websocket.SendHandler;
 import javax.websocket.SendResult;
 
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.net.AbstractEndpoint;
+import org.apache.tomcat.util.net.SocketWrapperBase;
 import org.apache.tomcat.util.res.StringManager;
 import org.apache.tomcat.websocket.Transformation;
 import org.apache.tomcat.websocket.WsRemoteEndpointImplBase;
 
 /**
  * This is the server side {@link javax.websocket.RemoteEndpoint} implementation
- * - i.e. what the server uses to send data to the client. Communication is over
- * a {@link ServletOutputStream}.
+ * - i.e. what the server uses to send data to the client.
  */
 public class WsRemoteEndpointImplServer extends WsRemoteEndpointImplBase {
 
     private static final StringManager sm =
-            StringManager.getManager(Constants.PACKAGE_NAME);
+            StringManager.getManager(WsRemoteEndpointImplServer.class);
     private static final Log log = LogFactory.getLog(WsRemoteEndpointImplServer.class);
 
-    private static final Queue<OnResultRunnable> onResultRunnables =
-            new ConcurrentLinkedQueue<>();
-
-    private final ServletOutputStream sos;
+    private final SocketWrapperBase<?> socketWrapper;
     private final WsWriteTimeout wsWriteTimeout;
-    private final ExecutorService executorService;
     private volatile SendHandler handler = null;
     private volatile ByteBuffer[] buffers = null;
 
     private volatile long timeoutExpiry = -1;
     private volatile boolean close;
 
-
-    public WsRemoteEndpointImplServer(ServletOutputStream sos, WsServerContainer serverContainer) {
-        this.sos = sos;
+    public WsRemoteEndpointImplServer(SocketWrapperBase<?> socketWrapper,
+            WsServerContainer serverContainer) {
+        this.socketWrapper = socketWrapper;
         this.wsWriteTimeout = serverContainer.getTimeout();
-        this.executorService = serverContainer.getExecutorService();
     }
 
 
@@ -72,12 +65,41 @@ public class WsRemoteEndpointImplServer extends WsRemoteEndpointImplBase {
 
 
     @Override
-    protected void doWrite(SendHandler handler, ByteBuffer... buffers) {
-        this.handler = handler;
-        this.buffers = buffers;
-        // This is definitely the same thread that triggered the write so a
-        // dispatch will be required.
-        onWritePossible(true);
+    protected void doWrite(SendHandler handler, long blockingWriteTimeoutExpiry,
+            ByteBuffer... buffers) {
+        if (blockingWriteTimeoutExpiry == -1) {
+            this.handler = handler;
+            this.buffers = buffers;
+            // This is definitely the same thread that triggered the write so a
+            // dispatch will be required.
+            onWritePossible(true);
+        } else {
+            // Blocking
+            for (ByteBuffer buffer : buffers) {
+                long timeout = blockingWriteTimeoutExpiry - System.currentTimeMillis();
+                if (timeout < 0) {
+                    SendResult sr = new SendResult(new SocketTimeoutException());
+                    handler.onResult(sr);
+                    return;
+                }
+                socketWrapper.setWriteTimeout(timeout);
+                try {
+                    socketWrapper.write(true, buffer);
+                    timeout = blockingWriteTimeoutExpiry - System.currentTimeMillis();
+                    if (timeout < 0) {
+                        SendResult sr = new SendResult(new SocketTimeoutException());
+                        handler.onResult(sr);
+                        return;
+                    }
+                    socketWrapper.setWriteTimeout(timeout);
+                    socketWrapper.flush(true);
+                    handler.onResult(SENDRESULT_OK);
+                } catch (IOException e) {
+                    SendResult sr = new SendResult(e);
+                    handler.onResult(sr);
+                }
+            }
+        }
     }
 
 
@@ -88,38 +110,40 @@ public class WsRemoteEndpointImplServer extends WsRemoteEndpointImplBase {
             // was written
             return;
         }
-        boolean complete = true;
+        boolean complete = false;
         try {
+            socketWrapper.flush(false);
             // If this is false there will be a call back when it is true
-            while (sos.isReady()) {
+            while (socketWrapper.isReadyForWrite()) {
                 complete = true;
                 for (ByteBuffer buffer : buffers) {
                     if (buffer.hasRemaining()) {
                         complete = false;
-                        sos.write(buffer.array(), buffer.arrayOffset(),
-                                buffer.limit());
-                        buffer.position(buffer.limit());
+                        socketWrapper.write(false, buffer);
                         break;
                     }
                 }
                 if (complete) {
-                    wsWriteTimeout.unregister(this);
-                    clearHandler(null, useDispatch);
-                    if (close) {
-                        close();
+                    socketWrapper.flush(false);
+                    complete = socketWrapper.isReadyForWrite();
+                    if (complete) {
+                        wsWriteTimeout.unregister(this);
+                        clearHandler(null, useDispatch);
+                        if (close) {
+                            close();
+                        }
                     }
                     break;
                 }
             }
-
-        } catch (IOException ioe) {
+        } catch (IOException | IllegalStateException e) {
             wsWriteTimeout.unregister(this);
-            clearHandler(ioe, useDispatch);
+            clearHandler(e, useDispatch);
             close();
         }
+
         if (!complete) {
             // Async write is in progress
-
             long timeout = getSendTimeout();
             if (timeout > 0) {
                 // Register with timeout thread
@@ -140,7 +164,7 @@ public class WsRemoteEndpointImplServer extends WsRemoteEndpointImplBase {
             clearHandler(new EOFException(), true);
         }
         try {
-            sos.close();
+            socketWrapper.close();
         } catch (IOException e) {
             if (log.isInfoEnabled()) {
                 log.info(sm.getString("wsRemoteEndpointServer.closeFailed"), e);
@@ -197,12 +221,12 @@ public class WsRemoteEndpointImplServer extends WsRemoteEndpointImplBase {
         buffers = null;
         if (sh != null) {
             if (useDispatch) {
-                OnResultRunnable r = onResultRunnables.poll();
-                if (r == null) {
-                    r = new OnResultRunnable(onResultRunnables);
-                }
-                r.init(sh, t);
-                if (executorService == null || executorService.isShutdown()) {
+                OnResultRunnable r = new OnResultRunnable(sh, t);
+                AbstractEndpoint<?> endpoint = socketWrapper.getEndpoint();
+                Executor containerExecutor = endpoint.getExecutor();
+                if (endpoint.isRunning() && containerExecutor != null) {
+                    containerExecutor.execute(r);
+                } else {
                     // Can't use the executor so call the runnable directly.
                     // This may not be strictly specification compliant in all
                     // cases but during shutdown only close messages are going
@@ -211,8 +235,6 @@ public class WsRemoteEndpointImplServer extends WsRemoteEndpointImplBase {
                     // 55715. The issues with nested calls was the reason for
                     // the separate thread requirement in the specification.
                     r.run();
-                } else {
-                    executorService.execute(r);
                 }
             } else {
                 if (t == null) {
@@ -227,16 +249,10 @@ public class WsRemoteEndpointImplServer extends WsRemoteEndpointImplBase {
 
     private static class OnResultRunnable implements Runnable {
 
-        private final Queue<OnResultRunnable> queue;
+        private final SendHandler sh;
+        private final Throwable t;
 
-        private volatile SendHandler sh;
-        private volatile Throwable t;
-
-        private OnResultRunnable(Queue<OnResultRunnable> queue) {
-            this.queue = queue;
-        }
-
-        private void init(SendHandler sh, Throwable t) {
+        private OnResultRunnable(SendHandler sh, Throwable t) {
             this.sh = sh;
             this.t = t;
         }
@@ -248,12 +264,6 @@ public class WsRemoteEndpointImplServer extends WsRemoteEndpointImplBase {
             } else {
                 sh.onResult(new SendResult(t));
             }
-            t = null;
-            sh = null;
-            // Return the Runnable to the queue when it has been finished with
-            // Note if this method takes an age to finish there shouldn't be any
-            // thread safety issues as the fields are cleared above.
-            queue.add(this);
         }
     }
 }
