@@ -41,7 +41,6 @@ import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
 import org.junit.Assert;
-import org.junit.Assume;
 import org.junit.Test;
 
 import org.apache.catalina.Context;
@@ -51,14 +50,10 @@ import org.apache.catalina.startup.TesterServlet;
 import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
 import org.apache.catalina.valves.TesterAccessLogValve;
-import org.apache.juli.logging.Log;
-import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.buf.ByteChunk;
 
 public class TestNonBlockingAPI extends TomcatBaseTest {
 
-    private static final Log log = LogFactory.getLog(TestNonBlockingAPI.class);
-
     private static final int CHUNK_SIZE = 1024 * 1024;
     private static final int WRITE_SIZE  = CHUNK_SIZE * 10;
     private static final byte[] DATA = new byte[WRITE_SIZE];
@@ -91,12 +86,6 @@ public class TestNonBlockingAPI extends TomcatBaseTest {
 
     @Test(expected=IOException.class)
     public void testNonBlockingReadIgnoreIsReady() throws Exception {
-        // TODO Investigate options to get this test to pass with the HTTP BIO
-        //      connector.
-        Assume.assumeFalse(
-                "Skipping as this test requires true non-blocking IO",
-                getTomcatInstance().getConnector().getProtocol()
-                        .equals("org.apache.coyote.http11.Http11Protocol"));
         doTestNonBlockingRead(true);
     }
 
@@ -125,6 +114,15 @@ public class TestNonBlockingAPI extends TomcatBaseTest {
 
     @Test
     public void testNonBlockingWrite() throws Exception {
+        testNonBlockingWriteInternal(false);
+    }
+
+    @Test
+    public void testNonBlockingWriteWithKeepAlive() throws Exception {
+        testNonBlockingWriteInternal(true);
+    }
+
+    private void testNonBlockingWriteInternal(boolean keepAlive) throws Exception {
         Tomcat tomcat = getTomcatInstance();
         // No file system docBase required
         Context ctx = tomcat.addContext("", null);
@@ -139,17 +137,25 @@ public class TestNonBlockingAPI extends TomcatBaseTest {
         SocketFactory factory = SocketFactory.getDefault();
         Socket s = factory.createSocket("localhost", getPort());
 
+        InputStream is = s.getInputStream();
+        byte[] buffer = new byte[8192];
+
         ByteChunk result = new ByteChunk();
+
         OutputStream os = s.getOutputStream();
+        if (keepAlive) {
+            os.write(("OPTIONS * HTTP/1.1\r\n" +
+                    "Host: localhost:" + getPort() + "\r\n" +
+                    "\r\n").getBytes(StandardCharsets.ISO_8859_1));
+            os.flush();
+            is.read(buffer);
+        }
         os.write(("GET / HTTP/1.1\r\n" +
                 "Host: localhost:" + getPort() + "\r\n" +
                 "Connection: close\r\n" +
                 "\r\n").getBytes(StandardCharsets.ISO_8859_1));
         os.flush();
 
-        InputStream is = s.getInputStream();
-        byte[] buffer = new byte[8192];
-
         int read = 0;
         int readSinceLastPause = 0;
         while (read != -1) {
@@ -175,7 +181,7 @@ public class TestNonBlockingAPI extends TomcatBaseTest {
         int lineStart = 0;
         int lineEnd = resultString.indexOf('\n', 0);
         String line = resultString.substring(lineStart, lineEnd + 1);
-        Assert.assertEquals("HTTP/1.1 200 OK\r\n", line);
+        Assert.assertEquals("HTTP/1.1 200 \r\n", line);
 
         // Check headers - looking to see if response is chunked (it should be)
         boolean chunked = false;
@@ -228,19 +234,19 @@ public class TestNonBlockingAPI extends TomcatBaseTest {
                 boolean found = false;
                 for (int i = totalBodyRead; i < (totalBodyRead + line.length()); i++) {
                     if (DATA[i] != resultBytes[lineStart + i - totalBodyRead]) {
-                        int dataStart = i - 16;
+                        int dataStart = i - 64;
                         if (dataStart < 0) {
                             dataStart = 0;
                         }
-                        int dataEnd = i + 16;
+                        int dataEnd = i + 64;
                         if (dataEnd > DATA.length) {
                             dataEnd = DATA.length;
                         }
-                        int resultStart = lineStart + i - totalBodyRead - 16;
+                        int resultStart = lineStart + i - totalBodyRead - 64;
                         if (resultStart < 0) {
                             resultStart = 0;
                         }
-                        int resultEnd = lineStart + i - totalBodyRead + 16;
+                        int resultEnd = lineStart + i - totalBodyRead + 64;
                         if (resultEnd > resultString.length()) {
                             resultEnd = resultString.length();
                         }
@@ -327,7 +333,7 @@ public class TestNonBlockingAPI extends TomcatBaseTest {
         int lineStart = 0;
         int lineEnd = resultString.indexOf('\n', 0);
         String line = resultString.substring(lineStart, lineEnd + 1);
-        Assert.assertEquals("HTTP/1.1 200 OK\r\n", line);
+        Assert.assertEquals("HTTP/1.1 200 \r\n", line);
 
         // Listeners are invoked and access valve entries created on a different
         // thread so give that thread a chance to complete its work.
@@ -499,25 +505,21 @@ public class TestNonBlockingAPI extends TomcatBaseTest {
                 @Override
                 public void onTimeout(AsyncEvent event) throws IOException {
                     log.info("onTimeout");
-
                 }
 
                 @Override
                 public void onStartAsync(AsyncEvent event) throws IOException {
                     log.info("onStartAsync");
-
                 }
 
                 @Override
                 public void onError(AsyncEvent event) throws IOException {
                     log.info("AsyncListener.onError");
-
                 }
 
                 @Override
                 public void onComplete(AsyncEvent event) throws IOException {
                     log.info("onComplete");
-
                 }
             });
             // step 2 - notify on read
