@@ -118,19 +118,21 @@ public class TestDefaultServlet extends TomcatBaseTest {
 
         tomcat.start();
 
-        TestGzipClient gzipClient = new TestGzipClient(getPort());
+        TestCompressedClient gzipClient = new TestCompressedClient(getPort());
 
         gzipClient.reset();
         gzipClient.setRequest(new String[] {
                 "GET /index.html HTTP/1.1" + CRLF +
                 "Host: localhost" + CRLF +
                 "Connection: Close" + CRLF +
-                "Accept-Encoding: gzip" + CRLF + CRLF });
+                "Accept-Encoding: gzip, br" + CRLF + CRLF });
         gzipClient.connect();
         gzipClient.processRequest();
         assertTrue(gzipClient.isResponse200());
         List<String> responseHeaders = gzipClient.getResponseHeaders();
+        assertTrue(responseHeaders.contains("Content-Encoding: gzip"));
         assertTrue(responseHeaders.contains("Content-Length: " + gzipSize));
+        assertTrue(responseHeaders.contains("Vary: accept-encoding"));
 
         gzipClient.reset();
         gzipClient.setRequest(new String[] {
@@ -144,6 +146,228 @@ public class TestDefaultServlet extends TomcatBaseTest {
         assertTrue(responseHeaders.contains("Content-Type: text/html"));
         assertFalse(responseHeaders.contains("Content-Encoding: gzip"));
         assertTrue(responseHeaders.contains("Content-Length: " + indexSize));
+        assertTrue(responseHeaders.contains("Vary: accept-encoding"));
+    }
+
+    /*
+     * Verify serving of brotli compressed resources from context root.
+     */
+    @Test
+    public void testBrotliCompressedFile() throws Exception {
+
+        Tomcat tomcat = getTomcatInstance();
+
+        File appDir = new File("test/webapp");
+
+        long brSize = new File(appDir, "index.html.br").length();
+        long indexSize = new File(appDir, "index.html").length();
+
+        // app dir is relative to server home
+        Context ctxt = tomcat.addContext("", appDir.getAbsolutePath());
+        Wrapper defaultServlet = Tomcat.addServlet(ctxt, "default",
+                "org.apache.catalina.servlets.DefaultServlet");
+        defaultServlet.addInitParameter("precompressed", "true");
+
+        ctxt.addServletMappingDecoded("/", "default");
+        ctxt.addMimeMapping("html", "text/html");
+
+        tomcat.start();
+
+        TestCompressedClient client = new TestCompressedClient(getPort());
+
+        client.reset();
+        client.setRequest(new String[] {
+                "GET /index.html HTTP/1.1" + CRLF +
+                        "Host: localhost" + CRLF +
+                        "Connection: Close" + CRLF +
+                        "Accept-Encoding: br, gzip" + CRLF + CRLF });
+        client.connect();
+        client.processRequest();
+        assertTrue(client.isResponse200());
+        List<String> responseHeaders = client.getResponseHeaders();
+        assertTrue(responseHeaders.contains("Content-Encoding: br"));
+        assertTrue(responseHeaders.contains("Content-Length: " + brSize));
+        assertTrue(responseHeaders.contains("Vary: accept-encoding"));
+
+        client.reset();
+        client.setRequest(new String[] {
+                "GET /index.html HTTP/1.1" + CRLF +
+                        "Host: localhost" + CRLF +
+                        "Connection: Close" + CRLF+ CRLF });
+        client.connect();
+        client.processRequest();
+        assertTrue(client.isResponse200());
+        responseHeaders = client.getResponseHeaders();
+        assertTrue(responseHeaders.contains("Content-Type: text/html"));
+        assertFalse(responseHeaders.contains("Content-Encoding"));
+        assertTrue(responseHeaders.contains("Content-Length: " + indexSize));
+        assertTrue(responseHeaders.contains("Vary: accept-encoding"));
+    }
+
+    /*
+     * Verify serving of custom compressed resources from context root.
+     */
+    @Test
+    public void testCustomCompressedFile() throws Exception {
+
+        Tomcat tomcat = getTomcatInstance();
+
+        File appDir = new File("test/webapp");
+
+        long brSize = new File(appDir, "index.html.br").length();
+        long gzSize = new File(appDir, "index.html.gz").length();
+
+        // app dir is relative to server home
+        Context ctxt = tomcat.addContext("", appDir.getAbsolutePath());
+        Wrapper defaultServlet = Tomcat.addServlet(ctxt, "default",
+                DefaultServlet.class.getName());
+        defaultServlet.addInitParameter("precompressed", "gzip=.gz,custom=.br");
+
+        ctxt.addServletMappingDecoded("/", "default");
+        ctxt.addMimeMapping("html", "text/html");
+
+        tomcat.start();
+
+        TestCompressedClient client = new TestCompressedClient(getPort());
+
+        client.reset();
+        client.setRequest(new String[] {
+                "GET /index.html HTTP/1.1" + CRLF +
+                        "Host: localhost" + CRLF +
+                        "Connection: Close" + CRLF +
+                        "Accept-Encoding: br, gzip ; q = 0.5 , custom" + CRLF + CRLF });
+        client.connect();
+        client.processRequest();
+        assertTrue(client.isResponse200());
+        List<String> responseHeaders = client.getResponseHeaders();
+        assertTrue(responseHeaders.contains("Content-Encoding: custom"));
+        assertTrue(responseHeaders.contains("Content-Length: " + brSize));
+        assertTrue(responseHeaders.contains("Vary: accept-encoding"));
+
+        client.reset();
+        client.setRequest(new String[] {
+                "GET /index.html HTTP/1.1" + CRLF +
+                        "Host: localhost" + CRLF +
+                        "Connection: Close" + CRLF +
+                        "Accept-Encoding: br;q=1,gzip,custom" + CRLF + CRLF });
+        client.connect();
+        client.processRequest();
+        assertTrue(client.isResponse200());
+        responseHeaders = client.getResponseHeaders();
+        assertTrue(responseHeaders.contains("Content-Encoding: gzip"));
+        assertTrue(responseHeaders.contains("Content-Length: " + gzSize));
+        assertTrue(responseHeaders.contains("Vary: accept-encoding"));
+    }
+
+    /*
+     * Verify that "*" and "identity" values are handled correctly in accept-encoding header.
+     */
+    @Test
+    public void testIdentityAndStarAcceptEncodings() throws Exception {
+
+        Tomcat tomcat = getTomcatInstance();
+
+        File appDir = new File("test/webapp");
+
+        long brSize = new File(appDir, "index.html.br").length();
+        long indexSize = new File(appDir, "index.html").length();
+
+        // app dir is relative to server home
+        Context ctxt = tomcat.addContext("", appDir.getAbsolutePath());
+        Wrapper defaultServlet = Tomcat.addServlet(ctxt, "default",
+                DefaultServlet.class.getName());
+        defaultServlet.addInitParameter("precompressed", "br=.br,gzip=.gz");
+
+        ctxt.addServletMappingDecoded("/", "default");
+        ctxt.addMimeMapping("html", "text/html");
+
+        tomcat.start();
+
+        TestCompressedClient client = new TestCompressedClient(getPort());
+
+        client.reset();
+        client.setRequest(new String[] {
+                "GET /index.html HTTP/1.1" + CRLF +
+                        "Host: localhost" + CRLF +
+                        "Connection: Close" + CRLF +
+                        "Accept-Encoding: gzip;q=0.9,*" + CRLF + CRLF });
+        client.connect();
+        client.processRequest();
+        assertTrue(client.isResponse200());
+        List<String> responseHeaders = client.getResponseHeaders();
+        assertTrue(responseHeaders.contains("Content-Encoding: br"));
+        assertTrue(responseHeaders.contains("Content-Length: " + brSize));
+        assertTrue(responseHeaders.contains("Vary: accept-encoding"));
+
+        client.reset();
+        client.setRequest(new String[] {
+                "GET /index.html HTTP/1.1" + CRLF +
+                        "Host: localhost" + CRLF +
+                        "Connection: Close" + CRLF +
+                        "Accept-Encoding: gzip;q=0.9,br;q=0,identity," + CRLF + CRLF });
+        client.connect();
+        client.processRequest();
+        assertTrue(client.isResponse200());
+        responseHeaders = client.getResponseHeaders();
+        assertFalse(responseHeaders.contains("Content-Encoding"));
+        assertTrue(responseHeaders.contains("Content-Length: " + indexSize));
+        assertTrue(responseHeaders.contains("Vary: accept-encoding"));
+    }
+
+    /*
+     * Verify preferring of brotli in default configuration for actual Firefox and Chrome requests.
+     */
+    @Test
+    public void testBrotliPreference() throws Exception {
+
+        Tomcat tomcat = getTomcatInstance();
+
+        File appDir = new File("test/webapp");
+
+        long brSize = new File(appDir, "index.html.br").length();
+
+        // app dir is relative to server home
+        Context ctxt = tomcat.addContext("", appDir.getAbsolutePath());
+        Wrapper defaultServlet = Tomcat.addServlet(ctxt, "default",
+                DefaultServlet.class.getName());
+        defaultServlet.addInitParameter("precompressed", "true");
+
+        ctxt.addServletMappingDecoded("/", "default");
+        ctxt.addMimeMapping("html", "text/html");
+
+        tomcat.start();
+
+        TestCompressedClient client = new TestCompressedClient(getPort());
+
+        // Firefox 45 Accept-Encoding
+        client.reset();
+        client.setRequest(new String[] {
+                "GET /index.html HTTP/1.1" + CRLF +
+                        "Host: localhost" + CRLF +
+                        "Connection: Close" + CRLF +
+                        "Accept-Encoding: gzip, deflate, br" + CRLF + CRLF });
+        client.connect();
+        client.processRequest();
+        assertTrue(client.isResponse200());
+        List<String> responseHeaders = client.getResponseHeaders();
+        assertTrue(responseHeaders.contains("Content-Encoding: br"));
+        assertTrue(responseHeaders.contains("Content-Length: " + brSize));
+        assertTrue(responseHeaders.contains("Vary: accept-encoding"));
+
+        // Chrome 50 Accept-Encoding
+        client.reset();
+        client.setRequest(new String[] {
+                "GET /index.html HTTP/1.1" + CRLF +
+                        "Host: localhost" + CRLF +
+                        "Connection: Close" + CRLF +
+                        "Accept-Encoding: gzip, deflate, sdch, br" + CRLF + CRLF });
+        client.connect();
+        client.processRequest();
+        assertTrue(client.isResponse200());
+        responseHeaders = client.getResponseHeaders();
+        assertTrue(responseHeaders.contains("Content-Encoding: br"));
+        assertTrue(responseHeaders.contains("Content-Length: " + brSize));
+        assertTrue(responseHeaders.contains("Vary: accept-encoding"));
     }
 
     /*
@@ -342,7 +566,7 @@ public class TestDefaultServlet extends TomcatBaseTest {
         assertTrue(client.isResponse404());
     }
 
-    /**
+    /*
      * Verifies that the same Content-Length is returned for both GET and HEAD
      * operations when a static resource served by the DefaultServlet is
      * included.
@@ -387,9 +611,9 @@ public class TestDefaultServlet extends TomcatBaseTest {
         }
     }
 
-    private static class TestGzipClient extends SimpleHttpClient {
+    private static class TestCompressedClient extends SimpleHttpClient {
 
-        public TestGzipClient(int port) {
+        public TestCompressedClient(int port) {
             setPort(port);
         }
 
