@@ -50,17 +50,15 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         // Has a protocol been specified
         String protocol = System.getProperty("tomcat.test.protocol");
 
-        // Use BIO by default
+        // Use NIO by default
         if (protocol == null) {
-            protocol = "org.apache.coyote.ajp.AjpProtocol";
+            protocol = "org.apache.coyote.ajp.AjpNioProtocol";
         } else if (protocol.contains("Nio2")) {
             protocol = "org.apache.coyote.ajp.AjpNio2Protocol";
-        } else if (protocol.contains("Nio")) {
-            protocol = "org.apache.coyote.ajp.AjpNioProtocol";
         } else if (protocol.contains("Apr")) {
             protocol = "org.apache.coyote.ajp.AjpAprProtocol";
         } else {
-            protocol = "org.apache.coyote.ajp.AjpProtocol";
+            protocol = "org.apache.coyote.ajp.AjpNioProtocol";
         }
 
         return protocol;
@@ -312,7 +310,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         }
 
         // Expect 3 packets: headers, body, end
-        validateResponseHeaders(responseHeaders, 200, "OK");
+        validateResponseHeaders(responseHeaders, 200, "200");
 
         String body = extractResponseBody(ajpClient.readMessage());
         RequestDescriptor result = SnoopResult.parse(body);
@@ -513,7 +511,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
 
         TesterAjpMessage responseHeaders = ajpClient.sendMessage(forwardMessage);
         // Expect 3 packets: headers, body, end
-        validateResponseHeaders(responseHeaders, 403, "Forbidden");
+        validateResponseHeaders(responseHeaders, 403, "403");
         //TesterAjpMessage responseBody = ajpClient.readMessage();
         //validateResponseBody(responseBody, HelloWorldServlet.RESPONSE_TEXT);
         validateResponseEnd(ajpClient.readMessage(), false);
@@ -527,7 +525,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
 
         responseHeaders = ajpClient.sendMessage(forwardMessage);
         // Expect 3 packets: headers, body, end
-        validateResponseHeaders(responseHeaders, 403, "Forbidden");
+        validateResponseHeaders(responseHeaders, 403, "403");
         //responseBody = ajpClient.readMessage();
         //validateResponseBody(responseBody, HelloWorldServlet.RESPONSE_TEXT);
         validateResponseEnd(ajpClient.readMessage(), false);
@@ -541,7 +539,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
 
         responseHeaders = ajpClient.sendMessage(forwardMessage);
         // Expect 3 packets: headers, body, end
-        validateResponseHeaders(responseHeaders, 200, "OK");
+        validateResponseHeaders(responseHeaders, 200, "200");
         TesterAjpMessage responseBody = ajpClient.readMessage();
         validateResponseBody(responseBody, HelloWorldServlet.RESPONSE_TEXT);
         validateResponseEnd(ajpClient.readMessage(), true);
@@ -578,7 +576,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         for (int i = 0; i < 2; i++) {
             TesterAjpMessage responseHeaders = ajpClient.sendMessage(forwardMessage);
             // Expect 3 packets: headers, body, end
-            validateResponseHeaders(responseHeaders, 200, "OK");
+            validateResponseHeaders(responseHeaders, 200, "200");
             TesterAjpMessage responseBody = ajpClient.readMessage();
             validateResponseBody(responseBody, HelloWorldServlet.RESPONSE_TEXT);
             validateResponseEnd(ajpClient.readMessage(), true);
@@ -595,14 +593,14 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
 
     @Test
     public void testPost() throws Exception {
-        doTestPost(false, HttpServletResponse.SC_OK, "OK");
+        doTestPost(false, HttpServletResponse.SC_OK, "200");
     }
 
 
     @Test
     public void testPostMultipleContentLength() throws Exception {
         // Multiple content lengths
-        doTestPost(true, HttpServletResponse.SC_BAD_REQUEST, "Bad Request");
+        doTestPost(true, HttpServletResponse.SC_BAD_REQUEST, "400");
     }
 
 
@@ -681,7 +679,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
                 ajpClient.sendMessage(forwardMessage, null);
 
         // Expect 2 messages: headers, end
-        validateResponseHeaders(responseHeaders, 304, "Not Modified");
+        validateResponseHeaders(responseHeaders, 304, "304");
         validateResponseEnd(ajpClient.readMessage(), true);
 
         // Double check the connection is still open
@@ -740,7 +738,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
                 ajpClient.sendMessage(forwardMessage, null);
 
         // Expect 3 messages: headers, body, end
-        validateResponseHeaders(responseHeaders, 200, "OK");
+        validateResponseHeaders(responseHeaders, 200, "200");
         validateResponseBody(ajpClient.readMessage(),
                 "Request Body length in bytes: 0");
         validateResponseEnd(ajpClient.readMessage(), true);
@@ -795,7 +793,7 @@ public class TestAbstractAjpProcessor extends TomcatBaseTest {
         TesterAjpMessage responseHeaders = ajpClient.sendMessage(forwardMessage);
 
         // Expect 3 messages: headers, body, end for a valid request
-        validateResponseHeaders(responseHeaders, 200, "OK");
+        validateResponseHeaders(responseHeaders, 200, "200");
         TesterAjpMessage responseBody = ajpClient.readMessage();
         Assert.assertTrue(responseBody.len > 15000);
         validateResponseEnd(ajpClient.readMessage(), true);
